import * as i0 from '@angular/core';
import { EventEmitter, Component, Input, Output, ViewChild, HostListener, Directive, NgModule } from '@angular/core';
import * as i1 from '@angular/common';
import { CommonModule } from '@angular/common';

const styleProperties = Object.freeze([
    'direction',
    'boxSizing',
    'width',
    'height',
    'overflowX',
    'overflowY',
    'borderTopWidth',
    'borderRightWidth',
    'borderBottomWidth',
    'borderLeftWidth',
    'borderStyle',
    'paddingTop',
    'paddingRight',
    'paddingBottom',
    'paddingLeft',
    // https://developer.mozilla.org/en-US/docs/Web/CSS/font
    'fontStyle',
    'fontVariant',
    'fontWeight',
    'fontStretch',
    'fontSize',
    'fontSizeAdjust',
    'lineHeight',
    'fontFamily',
    'textAlign',
    'textTransform',
    'textIndent',
    'textDecoration',
    'letterSpacing',
    'wordSpacing',
    'tabSize',
    'MozTabSize'
]);
const tagIndexIdPrefix = 'text-highlight-tag-id-';
function indexIsInsideTag(index, tag) {
    return tag.indices.start < index && index < tag.indices.end;
}
function overlaps(tagA, tagB) {
    return (indexIsInsideTag(tagB.indices.start, tagA) ||
        indexIsInsideTag(tagB.indices.end, tagA));
}
function isCoordinateWithinRect(rect, x, y) {
    return rect.left < x && x < rect.right && (rect.top < y && y < rect.bottom);
}
function escapeHtml(str) {
    return str.replace(/</g, '&lt;').replace(/>/g, '&gt;');
}
class TextInputHighlightComponent {
    constructor(renderer, cdr) {
        this.renderer = renderer;
        this.cdr = cdr;
        /**
         * The CSS class to add to highlighted tags
         */
        this.tagCssClass = '';
        /**
         * An array of indices of the textarea value to highlight
         */
        this.tags = [];
        /**
         * Called when the area over a tag is clicked
         */
        this.tagClick = new EventEmitter();
        /**
         * Called when the area over a tag is moused over
         */
        this.tagMouseEnter = new EventEmitter();
        /**
         * Called when the area over the tag has the mouse is removed from it
         */
        this.tagMouseLeave = new EventEmitter();
        /**
         * @private
         */
        this.highlightElementContainerStyle = {};
        this.textareaEventListeners = [];
        this.isDestroyed = false;
    }
    /**
     * Manually call this function to refresh the highlight element if the textarea styles have changed
     */
    refresh() {
        if (this.textInputElement) {
            const computed = getComputedStyle(this.textInputElement);
            styleProperties.forEach(prop => {
                this.highlightElementContainerStyle[prop] = computed[prop];
            });
        }
    }
    /**
     * @private
     */
    ngOnChanges(changes) {
        if (changes.textInputElement) {
            this.textInputElementChanged();
        }
        if (changes.tags || changes.tagCssClass || changes.textInputValue) {
            this.addTags();
        }
    }
    /**
     * @private
     */
    ngOnDestroy() {
        this.isDestroyed = true;
        this.textareaEventListeners.forEach(unregister => unregister());
    }
    /**
     * @private
     */
    onWindowResize() {
        this.refresh();
    }
    textInputElementChanged() {
        if (!this.textInputElement) {
            return;
        }
        const elementType = this.textInputElement.tagName.toLowerCase();
        if (elementType !== 'textarea') {
            throw new Error('The angular-text-input-highlight component must be passed ' +
                'a textarea to the `textInputElement` input. Instead received a ' +
                elementType);
        }
        setTimeout(() => {
            // in case the element was destroyed before the timeout fires
            if (!this.isDestroyed) {
                this.refresh();
                this.textareaEventListeners.forEach(unregister => unregister());
                this.textareaEventListeners = [
                    this.renderer.listen(this.textInputElement, 'input', () => {
                        this.addTags();
                    }),
                    this.renderer.listen(this.textInputElement, 'scroll', () => {
                        const highlightElement = this.highlightElement;
                        const textInputElement = this.textInputElement;
                        const highlightTagElements = this.highlightTagElements;
                        if (highlightElement && textInputElement && highlightTagElements) {
                            highlightElement.nativeElement.scrollTop = textInputElement.scrollTop;
                            this.highlightTagElements = highlightTagElements.map(tag => {
                                tag.clientRect = tag.element.getBoundingClientRect();
                                return tag;
                            });
                        }
                    }),
                    this.renderer.listen(this.textInputElement, 'mouseup', () => {
                        this.refresh();
                    })
                ];
                // only add event listeners if the host component actually asks for it
                if (this.tagClick.observers.length > 0) {
                    const onClick = this.renderer.listen(this.textInputElement, 'click', event => {
                        this.handleTextareaMouseEvent(event, 'click');
                    });
                    this.textareaEventListeners.push(onClick);
                }
                if (this.tagMouseEnter.observers.length > 0) {
                    const onMouseMove = this.renderer.listen(this.textInputElement, 'mousemove', event => {
                        this.handleTextareaMouseEvent(event, 'mousemove');
                    });
                    const onMouseLeave = this.renderer.listen(this.textInputElement, 'mouseleave', event => {
                        if (this.mouseHoveredTag) {
                            this.tagMouseLeave.emit(this.mouseHoveredTag);
                            this.mouseHoveredTag = undefined;
                        }
                    });
                    this.textareaEventListeners.push(onMouseMove);
                    this.textareaEventListeners.push(onMouseLeave);
                }
                this.addTags();
            }
        });
    }
    addTags() {
        if (!this.textInputElement || !this.textInputValue) {
            return;
        }
        const textInputValue = typeof this.textInputValue !== 'undefined'
            ? this.textInputValue
            : this.textInputElement.value;
        const prevTags = [];
        const parts = [];
        [...this.tags]
            .sort((tagA, tagB) => {
            return tagA.indices.start - tagB.indices.start;
        })
            .forEach(tag => {
            if (tag.indices.start > tag.indices.end) {
                throw new Error(`Highlight tag with indices [${tag.indices.start}, ${tag.indices
                    .end}] cannot start after it ends.`);
            }
            prevTags.forEach(prevTag => {
                if (overlaps(prevTag, tag)) {
                    throw new Error(`Highlight tag with indices [${tag.indices.start}, ${tag.indices
                        .end}] overlaps with tag [${prevTag.indices.start}, ${prevTag
                        .indices.end}]`);
                }
            });
            // TODO - implement this as an ngFor of items that is generated in the template for a cleaner solution
            const expectedTagLength = tag.indices.end - tag.indices.start;
            const tagContents = textInputValue.slice(tag.indices.start, tag.indices.end);
            if (tagContents.length === expectedTagLength) {
                const previousIndex = prevTags.length > 0 ? prevTags[prevTags.length - 1].indices.end : 0;
                const before = textInputValue.slice(previousIndex, tag.indices.start);
                parts.push(escapeHtml(before));
                const cssClass = tag.cssClass || this.tagCssClass;
                const tagId = tagIndexIdPrefix + this.tags.indexOf(tag);
                // text-highlight-tag-id-${id} is used instead of a data attribute to prevent an angular sanitization warning
                parts.push(`<span class="text-highlight-tag ${tagId} ${cssClass}">${escapeHtml(tagContents)}</span>`);
                prevTags.push(tag);
            }
        });
        const remainingIndex = prevTags.length > 0 ? prevTags[prevTags.length - 1].indices.end : 0;
        const remaining = textInputValue.slice(remainingIndex);
        parts.push(escapeHtml(remaining));
        parts.push('&nbsp;');
        this.highlightedText = parts.join('');
        this.cdr.detectChanges();
        if (this.highlightElement) {
            this.highlightTagElements = Array.from(this.highlightElement.nativeElement.getElementsByTagName('span')).map((elementUnknown) => {
                const element = elementUnknown;
                return { element, clientRect: element.getBoundingClientRect() };
            });
        }
    }
    handleTextareaMouseEvent(event, eventName) {
        if (!this.highlightTagElements) {
            return;
        }
        const matchingTagIndex = this.highlightTagElements.findIndex(elm => isCoordinateWithinRect(elm.clientRect, event.clientX, event.clientY));
        if (matchingTagIndex > -1) {
            const target = this.highlightTagElements[matchingTagIndex].element;
            const tagClass = Array.from(target.classList).find(className => className.startsWith(tagIndexIdPrefix));
            if (tagClass) {
                const tagId = tagClass.replace(tagIndexIdPrefix, '');
                const tag = this.tags[+tagId];
                const tagMouseEvent = { tag, target, event };
                if (eventName === 'click') {
                    this.tagClick.emit(tagMouseEvent);
                }
                else if (!this.mouseHoveredTag) {
                    this.mouseHoveredTag = tagMouseEvent;
                    this.tagMouseEnter.emit(tagMouseEvent);
                }
            }
        }
        else if (eventName === 'mousemove' && this.mouseHoveredTag) {
            this.mouseHoveredTag.event = event;
            this.tagMouseLeave.emit(this.mouseHoveredTag);
            this.mouseHoveredTag = undefined;
        }
    }
}
TextInputHighlightComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.13", ngImport: i0, type: TextInputHighlightComponent, deps: [{ token: i0.Renderer2 }, { token: i0.ChangeDetectorRef }], target: i0.ɵɵFactoryTarget.Component });
TextInputHighlightComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "12.0.0", version: "12.2.13", type: TextInputHighlightComponent, selector: "mwl-text-input-highlight", inputs: { tagCssClass: "tagCssClass", tags: "tags", textInputElement: "textInputElement", textInputValue: "textInputValue" }, outputs: { tagClick: "tagClick", tagMouseEnter: "tagMouseEnter", tagMouseLeave: "tagMouseLeave" }, host: { listeners: { "window:resize": "onWindowResize()" } }, viewQueries: [{ propertyName: "highlightElement", first: true, predicate: ["highlightElement"], descendants: true }], usesOnChanges: true, ngImport: i0, template: `
    <div
      class="text-highlight-element"
      [ngStyle]="highlightElementContainerStyle"
      [innerHtml]="highlightedText"
      #highlightElement>
    </div>
  `, isInline: true, directives: [{ type: i1.NgStyle, selector: "[ngStyle]", inputs: ["ngStyle"] }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.13", ngImport: i0, type: TextInputHighlightComponent, decorators: [{
            type: Component,
            args: [{
                    selector: 'mwl-text-input-highlight',
                    template: `
    <div
      class="text-highlight-element"
      [ngStyle]="highlightElementContainerStyle"
      [innerHtml]="highlightedText"
      #highlightElement>
    </div>
  `
                }]
        }], ctorParameters: function () { return [{ type: i0.Renderer2 }, { type: i0.ChangeDetectorRef }]; }, propDecorators: { tagCssClass: [{
                type: Input
            }], tags: [{
                type: Input
            }], textInputElement: [{
                type: Input
            }], textInputValue: [{
                type: Input
            }], tagClick: [{
                type: Output
            }], tagMouseEnter: [{
                type: Output
            }], tagMouseLeave: [{
                type: Output
            }], highlightElement: [{
                type: ViewChild,
                args: ['highlightElement']
            }], onWindowResize: [{
                type: HostListener,
                args: ['window:resize']
            }] } });

class TextInputHighlightContainerDirective {
}
TextInputHighlightContainerDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.13", ngImport: i0, type: TextInputHighlightContainerDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive });
TextInputHighlightContainerDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.2.13", type: TextInputHighlightContainerDirective, selector: "[mwlTextInputHighlightContainer]", host: { properties: { "class.text-input-highlight-container": "true" } }, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.13", ngImport: i0, type: TextInputHighlightContainerDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[mwlTextInputHighlightContainer]',
                    host: {
                        '[class.text-input-highlight-container]': 'true'
                    }
                }]
        }] });

class TextInputElementDirective {
}
TextInputElementDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.13", ngImport: i0, type: TextInputElementDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive });
TextInputElementDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "12.2.13", type: TextInputElementDirective, selector: "textarea[mwlTextInputElement]", host: { properties: { "class.text-input-element": "true" } }, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.13", ngImport: i0, type: TextInputElementDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: 'textarea[mwlTextInputElement]',
                    host: {
                        '[class.text-input-element]': 'true'
                    }
                }]
        }] });

class TextInputHighlightModule {
}
TextInputHighlightModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.13", ngImport: i0, type: TextInputHighlightModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
TextInputHighlightModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "12.2.13", ngImport: i0, type: TextInputHighlightModule, declarations: [TextInputHighlightComponent,
        TextInputHighlightContainerDirective,
        TextInputElementDirective], imports: [CommonModule], exports: [TextInputHighlightComponent,
        TextInputHighlightContainerDirective,
        TextInputElementDirective] });
TextInputHighlightModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "12.2.13", ngImport: i0, type: TextInputHighlightModule, imports: [[CommonModule]] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.13", ngImport: i0, type: TextInputHighlightModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [
                        TextInputHighlightComponent,
                        TextInputHighlightContainerDirective,
                        TextInputElementDirective
                    ],
                    imports: [CommonModule],
                    exports: [
                        TextInputHighlightComponent,
                        TextInputHighlightContainerDirective,
                        TextInputElementDirective
                    ]
                }]
        }] });

/*
 * Public API Surface of angular-text-input-highlight
 */

/**
 * Generated bundle index. Do not edit.
 */

export { TextInputElementDirective, TextInputHighlightComponent, TextInputHighlightContainerDirective, TextInputHighlightModule };
//# sourceMappingURL=anderstornkvist-angular-text-input-highlight.js.map
